/*
 * Definitions for packet ring
 *
 * (C) 2004 - nmon.net
 */

#ifndef _NCAP_H
#define _NCAP_H

#include <linux/types.h>

#define NCAP_VERSION               1
#define NCAP_ADAPTER_E1000         1

#define CONFIG_NCAP_TX

/* 
   This information is exported to userland
   via mmap()
*/

struct ncap_info {  
  u_char version /* 1 */, adapter_type; /* 1=e1000 */
  u_char mac_address[6];

  /* RX */
  unsigned long kernel_rx_info_len, kernel_rx_info_num;
  unsigned long kernel_rx_tot_mem, kernel_rx_slot_len, rx_reg_displ;
  unsigned int next_rx_slot;
  
  /* TX */
  unsigned long kernel_tx_info_len, kernel_tx_info_num;
  unsigned long kernel_tx_tot_mem, kernel_tx_slot_len, tx_reg_displ;
  unsigned int next_tx_slot;

  /* RX/TX */
  unsigned long ncap_adapter_memory_len;  
};

/* **************************** */

#ifndef PAGE_SIZE
#define PAGE_SIZE         4096
#endif

#define E1000_RXD_STAT_DD       0x01    /* Descriptor Done */

/* e1000_hw.h */
#define E1000_RDH      0x02810  /* RX Descriptor Head - RW */
#define E1000_RDT      0x02818  /* RX Descriptor Tail - RW */
#define E1000_TDT      0x03818  /* TX Descripotr Tail - RW */
#define E1000_TDH      0x03810  /* TX Descriptor Head - RW */

#define E1000_TXD_CMD_IFCS   0x02000000 /* Insert FCS (Ethernet CRC) */
#define E1000_TXD_CMD_IDE    0x80000000 /* Enable Tidv register */
#define E1000_TXD_CMD_EOP    0x01000000 /* End of Packet */
#define E1000_TXD_CMD_RS     0x08000000 /* Report Status */

#define E1000_RXD_STAT_DD       0x01    /* Descriptor Done */

/* ********************************* */

#ifndef __KERNEL__

struct e1000_rx_desc {
  u_int64_t buffer_addr; /* Address of the descriptor's data buffer */
  u_int16_t length;     /* Length of data DMAed into data buffer */
  u_int16_t csum;       /* Packet checksum */
  u_int8_t status;      /* Descriptor status */
  u_int8_t errors;      /* Descriptor Errors */
  u_int16_t special;
};

/* Transmit Descriptor */
struct e1000_tx_desc {
  u_int64_t buffer_addr;       /* Address of the descriptor's data buffer */
  union {
    u_int32_t data;
    struct {
      u_int16_t length;    /* Data buffer length */
      u_int8_t cso;        /* Checksum offset */
      u_int8_t cmd;        /* Descriptor control */
    } flags;
  } lower;
  union {
    u_int32_t data;
    struct {
      u_int8_t status;     /* Descriptor status */
      u_int8_t css;        /* Checksum start */
      u_int16_t special;
    } fields;
  } upper;
};

#endif /* __KERNEL__ */

/* ********************************* */

struct ncap_device_info {
  int32_t ncap_device_fd;
  struct e1000_rx_desc *kernel_rx_buffer;
  struct e1000_tx_desc *kernel_tx_buffer;
  unsigned char *write_register;
  unsigned char *kernel_rx_ring, *kernel_tx_ring;
  u_int32_t tx_next_idx, tx_cur_idx; /* TX */
  u_int32_t rx_cur_idx; /* RX */
  struct ncap_info *ncap_info;
  u_int32_t num_pkt_sent, num_pkt_rcvd;  
};

#endif /* _NCAP_H */
