/*
 *  Copyright (C) 2004
 *
 *  Luca Deri <deri@nmon.net>
 *  http://www.nmon.net/
 *
 */

#ifndef _NCAP_INT_H_
#define _NCAP_INT_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <signal.h>
#include <sys/io.h>
#include <time.h>
#include <syslog.h>

#define CONFIG_NCAP_TX
#include <linux/ncap.h>

/* ****************************************** */

/* nCap return codes */

/* 1. general return codes */
#define NCAP_NO_ERROR                      0
#define NCAP_ERR_NO_SUCH_DEVICE           -1
#define NCAP_ERR_MAPPING_FAILED           -2
#define NCAP_ERR_DEVICE_UNAVAILABLE       -3
#define NCAP_ERR_INVALID_LICENSE          -4
#define NCAP_INTERNAL_ERROR               -5

/* 2. TX return codes */
#define NCAP_TX_SLOT_NOT_YET_AVAILABLE    -10


/* ****************************************** */

/* init/term */
extern struct ncap_device_info *init_ncap_device(char *dev_name, int *error);
extern void term_ncap_device(struct ncap_device_info *dev);

#ifdef CONFIG_NCAP_TX
extern int ncap_send_packet(struct ncap_device_info *dev,
			    uint16_t packet_len, unsigned char *packet);
#endif

extern int ncap_recv_packet_available(struct ncap_device_info *dev,
				      int wait_for_packet);
extern int ncap_recv_packet(struct ncap_device_info *dev,
			    unsigned short *recv_packet_len,
			    unsigned char* packet_buffer,
			    unsigned short packet_buffer_len,
			    int wait_for_packet);

extern int ncap_set_if_promisc(const char *device, int set_promisc);
extern unsigned long long rdtsc();
extern void dump_rx_registers(struct ncap_device_info *dev);

/* ****************************************** */

#endif /* _NCAP_INT_H_ */
